/*
 * This is a generic implementation of the SLIP protocol over an RS232
 * (serial) device.
 */
#include <serial.h>
#include <time.h>
#include <string.h>
#if 0
#define puts(x)
#else
#include <stdio.h>
#endif
#include "uip.h"

#define SLIP_END     0300
#define SLIP_ESC     0333
#define SLIP_ESC_END 0334
#define SLIP_ESC_ESC 0335

#define MAX_SIZE UIP_BUFSIZE

static u8_t slip_buf[MAX_SIZE];

#if MAX_SIZE > 255
static u16_t len, tmplen;
#else
static u8_t len, tmplen;
#endif /* MAX_SIZE > 255 */

/*
 * rs232dev_send():
 *
 * Sends the packet in the uip_buf and uip_appdata buffers. The first
 * 40 bytes of the packet (the IP and TCP headers) are read from the
 * uip_buf buffer, and the following bytes (the application data) are
 * read from the uip_appdata buffer.
 */
void rs232dev_send(void)
{
#if MAX_SIZE > 255
   u16_t i;
#else
   u8_t i;
#endif /* MAX_SIZE > 255 */
   u8_t *ptr;
   u8_t c;

   ComSendChar(SLIP_END);
   ptr = uip_buf;
   for (i = 0; i < uip_len; ++i)
   {
      if (i == 40)
      {
	 ptr = (u8_t *)uip_appdata;
      }
      c = *ptr++;
      switch (c)
      {
	 case SLIP_END:
	    ComSendChar(SLIP_ESC);
	    ComSendChar(SLIP_ESC_END);
	    break;
	 case SLIP_ESC:
	    ComSendChar(SLIP_ESC);
	    ComSendChar(SLIP_ESC_ESC);
	    break;
	 default:
	    ComSendChar(c);
	    break;
      }
   }
   ComSendChar(SLIP_END);
}

/*
 * rs232dev_poll():
 *
 * Read all avaliable bytes from the RS232 interface into the slip_buf
 * buffer. If no more bytes are avaliable, it returns with 0 to
 * indicate that no packet was immediately ready. When a full packet
 * has been read into the buffer, the packet is copied into the
 * uip_buf buffer and the length of the packet is returned.
 */
#if MAX_SIZE > 255
u16_t
#else 
u8_t
#endif /* MAX_SIZE > 255 */
rs232dev_poll(void)
{
   u8_t c;
   static u8_t lastc;
   int com_ret;
  
   while ((com_ret = ComRecChar()) != -1)
   {
      c = (u8_t)com_ret;
      switch (c)
      {
	 case SLIP_ESC:
	    lastc = c;
	    break;
	 case SLIP_END:
	    lastc = c;
	    /* End marker found, we copy our input buffer to the uip_buf
	       buffer and return the size of the packet we copied. */
	    memcpy(uip_buf, slip_buf, len);
	    tmplen = len;
	    len = 0;
	    return tmplen;
	 default:     
	    if (lastc == SLIP_ESC)
	    {
	       lastc = c;
	       /* Previous read byte was an escape byte, so this byte will 
		  be interpreted differently from others. */
	       switch (c)
	       {
		  case SLIP_ESC_END:
		     c = SLIP_END;
		     break;
		  case SLIP_ESC_ESC:
		     c = SLIP_ESC;
		     break;
	       }
	    }
	    else
	    {
	       lastc = c;
	    }
	    slip_buf[len] = c;
	    ++len;
	    if (len > MAX_SIZE)
	    {
	       len = 0;
	    }
	    break;
      }
   }
   return 0;
}

/*
 * rs232dev_init():
 *
 * Initializes the RS232 device and sets the parameters of the device.
 */ 
void rs232dev_init(void)
{
   if (OpenComPort( COM_PORT_1 ) != 0)
      puts("RS232 not initialized\n");
   else
      puts("RS232 OK\n");
   InitComPort( COM_BAUD_9600, COM_DATABITS_8, 
		COM_PARITY_NO, COM_STOPBITS_1 );
   len = 0;
}

/*
 * rs232dev_close():
 *
 * Closes the RS232 device.
 */ 
void rs232dev_close(void)
{
   CloseComPort();
}
